"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonRxServer = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const util_1 = require("./util");
class JsonRxServer {
    constructor({ send, call, notify, maxActiveSubscriptions = 30 }) {
        this.active = new Map();
        this.send = send;
        this.call = call;
        this.notify = notify;
        this.maxActiveSubscriptions = maxActiveSubscriptions;
    }
    sendError(id, error) {
        const message = [-1, id, error];
        this.send(message);
    }
    onSubscribe(message) {
        const [id, name, payload] = message;
        util_1.assertId(id);
        util_1.assertName(name);
        try {
            const active = this.active.get(id);
            if (active) {
                this.active.delete(id);
                active.unsubscribe();
                const message = [-1, id, { message: 'ID already active.' }];
                this.send(message);
                return;
            }
            if (this.active.size >= this.maxActiveSubscriptions)
                return this.sendError(id, { message: 'Too many subscriptions.' });
            const callResult = this.call(name, payload);
            const observable = rxjs_1.isObservable(callResult)
                ? callResult
                : rxjs_1.from(callResult).pipe(operators_1.mergeMap(value => rxjs_1.isObservable(value) ? value : rxjs_1.of(value)));
            const ref = { buffer: [] };
            let done = false;
            const subscription = observable.subscribe((data) => {
                if (data === undefined)
                    return;
                ref.buffer.push(data);
                util_1.microtask(() => {
                    if (!ref.buffer.length)
                        return;
                    try {
                        for (const payload of ref.buffer)
                            this.send([-2, id, payload]);
                    }
                    finally {
                        ref.buffer = [];
                    }
                });
            }, (error) => {
                done = true;
                this.active.delete(id);
                const data = error instanceof Error ? { message: error.message } : error;
                const message = [-1, id, data];
                this.send(message);
            }, () => {
                done = true;
                this.active.delete(id);
                try {
                    if (!ref.buffer.length)
                        return this.send([0, id]);
                    const last = ref.buffer.length - 1;
                    for (let i = 0; i <= last; i++) {
                        const isLast = i === last;
                        const message = isLast ? [0, id, ref.buffer[i]] : [-2, id, ref.buffer[i]];
                        this.send(message);
                    }
                }
                finally {
                    ref.buffer = [];
                }
            });
            if (!done)
                this.active.set(id, subscription);
        }
        catch (error) {
            this.sendError(id, error instanceof Error ? { message: error.message } : error);
        }
    }
    onUnsubscribe(message) {
        const [, id] = message;
        util_1.assertId(id);
        const subscription = this.active.get(id);
        if (!subscription)
            return;
        this.active.delete(id);
        subscription.unsubscribe();
    }
    onNotification(message) {
        const [name, payload] = message;
        util_1.assertName(name);
        this.notify(name, payload);
    }
    onMessage(message) {
        if (!util_1.isArray(message))
            throw new Error('Invalid message');
        const [one] = message;
        if (typeof one === 'string')
            return this.onNotification(message);
        if (one > 0)
            return this.onSubscribe(message);
        if (one === -3)
            return this.onUnsubscribe(message);
        throw new Error('Invalid message');
    }
}
exports.JsonRxServer = JsonRxServer;
